==> jhipster-core-1.4.5.tgz: package/lib/parser/json_parser.js <==
/**
 * Copyright 2013-2018 the original author or authors from the JHipster project.
 *
 * This file is part of the JHipster project, see http://www.jhipster.tech/
 * for more information.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// todo use JSONEntity in this file, and further refactor the code

const _ = require('lodash');
const JDLObject = require('../core/jdl_object');
const JDLEntity = require('../core/jdl_entity');
const JDLField = require('../core/jdl_field');
const JDLValidation = require('../core/jdl_validation');
const JDLEnum = require('../core/jdl_enum');
const JDLRelationship = require('../core/jdl_relationship');
const JDLUnaryOption = require('../core/jdl_unary_option');
const JDLBinaryOption = require('../core/jdl_binary_option');
const FieldTypes = require('../core/jhipster/field_types');
const RelationshipTypes = require('../core/jhipster/relationship_types').RELATIONSHIP_TYPES;
const UnaryOptions = require('../core/jhipster/unary_options').UNARY_OPTIONS;
const BinaryOptions = require('../core/jhipster/binary_options').BINARY_OPTIONS;
const BuildException = require('../exceptions/exception_factory').BuildException;
const exceptions = require('../exceptions/exception_factory').exceptions;

module.exports = {
  parseEntities,
  parseServerOptions
};

const USER = 'User';
const USER_ENTITY = new JDLEntity({ name: USER });

const configuration = {
  entities: null,
  jdl: new JDLObject(),
  skipUserManagement: false
};

/**
 * Converts the JSON entities into JDL.
 * @param entities the entities in plain JS format.
 * @param jdl {JDLObject} the base JDLObject to use as conversion result.
 * @returns {*} the updated JDLObject.
 */
function parseEntities(entities, jdl) {
  if (!entities) {
    throw new BuildException(exceptions.NullPointer, 'Entities have to be passed to be converted.');
  }
  init(entities, jdl);
  addEntities();
  addRelationshipsToJDL();
  return configuration.jdl;
}

function init(entities, jdl) {
  configuration.entities = entities;
  configuration.jdl = jdl || configuration.jdl;
  configuration.skipUserManagement = hasSkipUserManagement();
}

function hasSkipUserManagement() {
  return configuration.jdl.options.has(UnaryOptions.SKIP_USER_MANAGEMENT);
}

function addEntities() {
  for (let i = 0, entityNames = Object.keys(configuration.entities); i < entityNames.length; i++) {
    addEntity(entityNames[i]);
  }
}

function addEntity(entityName) {
  if (entityName === USER && !configuration.skipUserManagement) {
    throw new BuildException(
      exceptions.IllegalName,
      'User entity name is reserved if skipUserManagement is not set.');
  }
  const entity = configuration.entities[entityName];
  configuration.jdl.addEntity(convertJSONToJDLEntity(entity, entityName));
  addEnumsToJDL(entity);
  addEntityOptionsToJDL(entity, entityName);
}

function convertJSONToJDLEntity(entity, entityName) {
  const jdlEntity = new JDLEntity({
    name: entityName,
    tableName: entity.entityTableName,
    comment: entity.javadoc
  });
  addFields(jdlEntity, entity);
  return jdlEntity;
}

function addFields(jdlEntity, entity) {
  entity.fields.forEach((field) => {
    jdlEntity.addField(convertJSONToJDLField(field));
  });
}

function convertJSONToJDLField(field) {
  const jdlField = new JDLField({
    name: _.lowerFirst(field.fieldName),
    type: field.fieldType,
    comment: field.javadoc
  });
  if (jdlField.type === 'byte[]') {
    jdlField.type = getTypeForBlob(field.fieldTypeBlobContent);
  }
  if (field.fieldValidateRules) {
    addValidations(jdlField, field);
  }
  return jdlField;
}

function getTypeForBlob(fieldTypeBlobContent) {
  if (['image', 'any', 'text'].includes(fieldTypeBlobContent)) {
    return FieldTypes.COMMON_DB_TYPES[`${fieldTypeBlobContent.toUpperCase()}_BLOB`];
  }
  return null;
}

function addValidations(jdlField, field) {
  field.fieldValidateRules.forEach((rule) => {
    jdlField.addValidation(convertJSONToJDLValidation(rule, field));
  });
}

function convertJSONToJDLValidation(rule, field) {
  return new JDLValidation({
    name: rule,
    value: field[`fieldValidateRules${_.upperFirst(rule)}`]
  });
}

function addEnumsToJDL(entity) {
  entity.fields.forEach((field) => {
    if (field.fieldValues !== undefined) {
      configuration.jdl.addEnum(new JDLEnum({
        name: field.fieldType,
        values: field.fieldValues.split(',')
      }));
    }
  });
}

/*
 * Adds relationships for entities to JDL.
 * The jdl passed must contain the jdl entities concerned by the relationships
 */
function addRelationshipsToJDL() {
  for (let i = 0, entityNames = Object.keys(configuration.entities); i < entityNames.length; i++) {
    dealWithRelationships(
      configuration.entities[entityNames[i]].relationships,
      entityNames[i]
    );
  }
}

function dealWithRelationships(relationships, entityName) {
  relationships.forEach((relationship) => {
    const jdlRelationship = getRelationship(relationship, entityName);
    if (jdlRelationship) {
      configuration.jdl.addRelationship(jdlRelationship);
    }
  });
}

function getRelationship(relationship, entityName) {
  const relationshipConfiguration = {
    type: null,
    injectedFieldInTo: null,
    isInjectedFieldInToRequired: false,
    commentInTo: null,
    otherEntity: null,
    toJdlEntity: null
  };

  if (shouldHandleUserEntity(relationship)) {
    relationshipConfiguration.toJdlEntity = USER_ENTITY;
  } else {
    relationshipConfiguration.otherEntity = configuration.entities[_.upperFirst(relationship.otherEntityName)];
    if (!relationshipConfiguration.otherEntity) {
      return;
    }
    setUpRelationshipConfiguration(relationship, entityName, relationshipConfiguration);
  }
  if (relationship.relationshipType === 'many-to-one') {
    if (relationshipConfiguration.injectedFieldInTo) {
      // This is a bidirectional relationship so consider it as a OneToMany
      // eslint-disable-next-line consistent-return
      return getBidirectionalOneToManyRelationship(relationship, entityName, relationshipConfiguration);
    }
    // Unidirectional ManyToOne
    relationshipConfiguration.type = RelationshipTypes.MANY_TO_ONE;
  } else if (relationship.relationshipType === 'one-to-one' && relationship.ownerSide === true) {
    relationshipConfiguration.type = RelationshipTypes.ONE_TO_ONE;
  } else if (relationship.relationshipType === 'many-to-many' && relationship.ownerSide === true) {
    relationshipConfiguration.type = RelationshipTypes.MANY_TO_MANY;
  }
  if (relationshipConfiguration.type) {
    // eslint-disable-next-line consistent-return
    return getDefaultRelationship(relationship, entityName, relationshipConfiguration);
  }
}

function shouldHandleUserEntity(relationship) {
  return relationship.otherEntityName.toLowerCase() === USER.toLowerCase() && !configuration.skipUserManagement;
}

function setUpRelationshipConfiguration(relationship, entityName, relationshipConfiguration) {
  relationshipConfiguration.toJdlEntity = configuration.jdl.entities[_.upperFirst(relationship.otherEntityName)];
  for (let i = 0; i < relationshipConfiguration.otherEntity.relationships.length; i++) {
    const relationship2 = relationshipConfiguration.otherEntity.relationships[i];
    if (_.upperFirst(relationship2.otherEntityName) === entityName
      && relationship2.otherEntityRelationshipName === relationship.relationshipName) {
      // Bidirectional relationship
      relationshipConfiguration.injectedFieldInTo = relationship2.relationshipName;
      if (!!relationship2.otherEntityField && relationship2.otherEntityField !== 'id') {
        relationshipConfiguration.injectedFieldInTo += `(${relationship2.otherEntityField})`;
      }
      relationshipConfiguration.isInjectedFieldInToRequired = !!relationship2.relationshipValidateRules;
      relationshipConfiguration.commentInTo = relationship2.javadoc;
      break;
    }
  }
}

function getDefaultRelationship(relationship, entityName, relationshipConfiguration) {
  return new JDLRelationship({ // eslint-disable-line consistent-return
    from: configuration.jdl.entities[entityName],
    to: relationshipConfiguration.toJdlEntity,
    type: relationshipConfiguration.type,
    injectedFieldInFrom: getInjectedFieldInFrom(relationship),
    injectedFieldInTo: relationshipConfiguration.injectedFieldInTo,
    isInjectedFieldInFromRequired: relationship.relationshipValidateRules,
    isInjectedFieldInToRequired: relationshipConfiguration.isInjectedFieldInToRequired,
    commentInFrom: relationship.javadoc,
    commentInTo: relationshipConfiguration.commentInTo
  });
}

function getBidirectionalOneToManyRelationship(relationship, entityName, relationshipConfiguration) {
  return new JDLRelationship({
    from: configuration.jdl.entities[_.upperFirst(relationship.otherEntityName)],
    to: configuration.jdl.entities[entityName],
    type: RelationshipTypes.ONE_TO_MANY,
    injectedFieldInFrom: relationshipConfiguration.injectedFieldInTo,
    injectedFieldInTo: getInjectedFieldInFrom(relationship),
    isInjectedFieldInFromRequired: relationshipConfiguration.isInjectedFieldInToRequired,
    isInjectedFieldInToRequired: relationship.relationshipValidateRules,
    commentInFrom: relationshipConfiguration.commentInTo,
    commentInTo: relationship.javadoc
  });
}

function getInjectedFieldInFrom(relationship) {
  return relationship.relationshipName
    + (relationship.otherEntityField && relationship.otherEntityField !== 'id'
      ? `(${relationship.otherEntityField})`
      : '');
}

function addEntityOptionsToJDL(entity, entityName) {
  if (entity.fluentMethods === false) {
    addUnaryOptionToJDL(UnaryOptions.NO_FLUENT_METHOD, entityName);
  }
  [BinaryOptions.DTO, BinaryOptions.PAGINATION, BinaryOptions.SERVICE, BinaryOptions.SEARCH_ENGINE].forEach((option) => {
    if (entity[option] && entity[option] !== 'no') {
      addBinaryOptionToJDL(option, entity[option], entityName);
    }
  });
  // angularSuffix in BINARY_OPTIONS, angularJSSuffix in Json
  if (entity.angularJSSuffix) {
    addBinaryOptionToJDL(BinaryOptions.ANGULAR_SUFFIX, entity.angularJSSuffix, entityName);
  }
  // microservice in BINARY_OPTIONS, microserviceName in Json
  if (entity.microserviceName !== undefined) {
    addBinaryOptionToJDL(BinaryOptions.MICROSERVICE, entity.microserviceName, entityName);
  }
  if (entity.jpaMetamodelFiltering === true) {
    addUnaryOptionToJDL(UnaryOptions.FILTER, entityName);
  }
  if (entity.clientRootFolder) {
    addBinaryOptionToJDL(BinaryOptions.CLIENT_ROOT_FOLDER, entity.clientRootFolder, entityName);
  }
}

function addUnaryOptionToJDL(unaryOption, entityName) {
  configuration.jdl.addOption(
    new JDLUnaryOption(
      {
        name: unaryOption,
        entityNames: [entityName]
      }
    )
  );
}

function addBinaryOptionToJDL(binaryOption, value, entityName) {
  configuration.jdl.addOption(
    new JDLBinaryOption(
      {
        name: binaryOption,
        value,
        entityNames: [entityName]
      }
    )
  );
}

/**
 * Parses the jhipster configuration into JDL.
 * @param jhConfig the jhipster config ('generator-jhipster' in .yo-rc.json)
 * @param jdl {JDLObject} to which the parsed options are added. If undefined a new JDLObject is created.
 * @returns {JDLObject} the JDLObject
 */
function parseServerOptions(jhConfig, jdl) {
  if (!jdl) {
    jdl = new JDLObject();
  }
  [UnaryOptions.SKIP_CLIENT, UnaryOptions.SKIP_SERVER, UnaryOptions.SKIP_USER_MANAGEMENT].forEach((option) => {
    if (jhConfig[option] === true) {
      jdl.addOption(new JDLUnaryOption({
        name: option,
        value: jhConfig[option]
      }));
    }
  });
  return jdl;
}

==> jhipster-core-2.0.2.tgz: package/lib/parser/json_parser.js <==
/**
 * Copyright 2013-2018 the original author or authors from the JHipster project.
 *
 * This file is part of the JHipster project, see http://www.jhipster.tech/
 * for more information.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

const _ = require('lodash');
const JDLObject = require('../core/jdl_object');
const JDLEntity = require('../core/jdl_entity');
const JDLField = require('../core/jdl_field');
const JDLValidation = require('../core/jdl_validation');
const JDLEnum = require('../core/jdl_enum');
const JDLRelationship = require('../core/jdl_relationship');
const JDLUnaryOption = require('../core/jdl_unary_option');
const JDLBinaryOption = require('../core/jdl_binary_option');
const FieldTypes = require('../core/jhipster/field_types');
const RelationshipTypes = require('../core/jhipster/relationship_types');
const UnaryOptions = require('../core/jhipster/unary_options');
const BinaryOptions = require('../core/jhipster/binary_options').Options;
const BuildException = require('../exceptions/exception_factory').BuildException;
const exceptions = require('../exceptions/exception_factory').exceptions;

module.exports = {
  parseEntities,
  parseServerOptions
};

const USER = 'User';
const USER_ENTITY = new JDLEntity({ name: USER });

const configuration = {
  entities: null,
  jdl: new JDLObject(),
  skipUserManagement: false
};

/**
 * Converts the JSON entities into JDL.
 * @param entities the entities in plain JS format.
 * @param jdl {JDLObject} the base JDLObject to use as conversion result.
 * @returns {*} the updated JDLObject.
 */
function parseEntities(entities, jdl) {
  if (!entities) {
    throw new BuildException(exceptions.NullPointer, 'Entities have to be passed to be converted.');
  }
  init(entities, jdl);
  addEntities();
  addRelationshipsToJDL();
  return configuration.jdl;
}

function init(entities, jdl) {
  configuration.entities = entities;
  configuration.jdl = jdl || configuration.jdl;
  configuration.skipUserManagement = hasSkipUserManagement();
}

function hasSkipUserManagement() {
  return configuration.jdl.options.has(UnaryOptions.SKIP_USER_MANAGEMENT);
}

function addEntities() {
  for (let i = 0, entityNames = Object.keys(configuration.entities); i < entityNames.length; i++) {
    addEntity(entityNames[i]);
  }
}

function addEntity(entityName) {
  if (entityName === USER && !configuration.skipUserManagement) {
    throw new BuildException(
      exceptions.IllegalName,
      'User entity name is reserved if skipUserManagement is not set.');
  }
  const entity = configuration.entities[entityName];
  configuration.jdl.addEntity(convertJSONToJDLEntity(entity, entityName));
  addEnumsToJDL(entity);
  addEntityOptionsToJDL(entity, entityName);
}

function convertJSONToJDLEntity(entity, entityName) {
  const jdlEntity = new JDLEntity({
    name: entityName,
    tableName: entity.entityTableName,
    comment: entity.javadoc
  });
  addFields(jdlEntity, entity);
  return jdlEntity;
}

function addFields(jdlEntity, entity) {
  entity.fields.forEach((field) => {
    jdlEntity.addField(convertJSONToJDLField(field));
  });
}

function convertJSONToJDLField(field) {
  const jdlField = new JDLField({
    name: _.lowerFirst(field.fieldName),
    type: field.fieldType,
    comment: field.javadoc
  });
  if (jdlField.type === 'byte[]') {
    jdlField.type = getTypeForBlob(field.fieldTypeBlobContent);
  }
  if (field.fieldValidateRules) {
    addValidations(jdlField, field);
  }
  return jdlField;
}

function getTypeForBlob(fieldTypeBlobContent) {
  if (['image', 'any', 'text'].includes(fieldTypeBlobContent)) {
    return FieldTypes.CommonDBTypes[`${fieldTypeBlobContent.toUpperCase()}_BLOB`];
  }
  return null;
}

function addValidations(jdlField, field) {
  field.fieldValidateRules.forEach((rule) => {
    jdlField.addValidation(convertJSONToJDLValidation(rule, field));
  });
}

function convertJSONToJDLValidation(rule, field) {
  return new JDLValidation({
    name: rule,
    value: field[`fieldValidateRules${_.upperFirst(rule)}`]
  });
}

function addEnumsToJDL(entity) {
  entity.fields.forEach((field) => {
    if (field.fieldValues !== undefined) {
      configuration.jdl.addEnum(new JDLEnum({
        name: field.fieldType,
        values: field.fieldValues.split(',')
      }));
    }
  });
}

/*
 * Adds relationships for entities to JDL.
 * The jdl passed must contain the jdl entities concerned by the relationships
 */
function addRelationshipsToJDL() {
  for (let i = 0, entityNames = Object.keys(configuration.entities); i < entityNames.length; i++) {
    dealWithRelationships(
      configuration.entities[entityNames[i]].relationships,
      entityNames[i]
    );
  }
}

function dealWithRelationships(relationships, entityName) {
  relationships.forEach((relationship) => {
    const jdlRelationship = getRelationship(relationship, entityName);
    if (jdlRelationship) {
      configuration.jdl.addRelationship(jdlRelationship);
    }
  });
}

function getRelationship(relationship, entityName) {
  const relationshipConfiguration = {
    type: null,
    injectedFieldInTo: null,
    isInjectedFieldInToRequired: false,
    commentInTo: null,
    otherEntity: null,
    toJdlEntity: null
  };

  if (shouldHandleUserEntity(relationship)) {
    relationshipConfiguration.toJdlEntity = USER_ENTITY;
  } else {
    relationshipConfiguration.otherEntity = configuration.entities[_.upperFirst(relationship.otherEntityName)];
    if (!relationshipConfiguration.otherEntity) {
      return;
    }
    setUpRelationshipConfiguration(relationship, entityName, relationshipConfiguration);
  }
  if (relationship.relationshipType === 'many-to-one') {
    if (relationshipConfiguration.injectedFieldInTo) {
      // This is a bidirectional relationship so consider it as a OneToMany
      // eslint-disable-next-line consistent-return
      return getBidirectionalOneToManyRelationship(relationship, entityName, relationshipConfiguration);
    }
    // Unidirectional ManyToOne
    relationshipConfiguration.type = RelationshipTypes.MANY_TO_ONE;
  } else if (relationship.relationshipType === 'one-to-one' && relationship.ownerSide === true) {
    relationshipConfiguration.type = RelationshipTypes.ONE_TO_ONE;
  } else if (relationship.relationshipType === 'many-to-many' && relationship.ownerSide === true) {
    relationshipConfiguration.type = RelationshipTypes.MANY_TO_MANY;
  }
  if (relationshipConfiguration.type) {
    // eslint-disable-next-line consistent-return
    return getDefaultRelationship(relationship, entityName, relationshipConfiguration);
  }
}

function shouldHandleUserEntity(relationship) {
  return relationship.otherEntityName.toLowerCase() === USER.toLowerCase() && !configuration.skipUserManagement;
}

function setUpRelationshipConfiguration(relationship, entityName, relationshipConfiguration) {
  relationshipConfiguration.toJdlEntity = configuration.jdl.entities[_.upperFirst(relationship.otherEntityName)];
  for (let i = 0; i < relationshipConfiguration.otherEntity.relationships.length; i++) {
    const relationship2 = relationshipConfiguration.otherEntity.relationships[i];
    if (_.upperFirst(relationship2.otherEntityName) === entityName
      && relationship2.otherEntityRelationshipName === relationship.relationshipName) {
      // Bidirectional relationship
      relationshipConfiguration.injectedFieldInTo = relationship2.relationshipName;
      if (!!relationship2.otherEntityField && relationship2.otherEntityField !== 'id') {
        relationshipConfiguration.injectedFieldInTo += `(${relationship2.otherEntityField})`;
      }
      relationshipConfiguration.isInjectedFieldInToRequired = !!relationship2.relationshipValidateRules;
      relationshipConfiguration.commentInTo = relationship2.javadoc;
      break;
    }
  }
}

function getDefaultRelationship(relationship, entityName, relationshipConfiguration) {
  return new JDLRelationship({ // eslint-disable-line consistent-return
    from: configuration.jdl.entities[entityName],
    to: relationshipConfiguration.toJdlEntity,
    type: relationshipConfiguration.type,
    injectedFieldInFrom: getInjectedFieldInFrom(relationship),
    injectedFieldInTo: relationshipConfiguration.injectedFieldInTo,
    isInjectedFieldInFromRequired: relationship.relationshipValidateRules,
    isInjectedFieldInToRequired: relationshipConfiguration.isInjectedFieldInToRequired,
    commentInFrom: relationship.javadoc,
    commentInTo: relationshipConfiguration.commentInTo
  });
}

function getBidirectionalOneToManyRelationship(relationship, entityName, relationshipConfiguration) {
  return new JDLRelationship({
    from: configuration.jdl.entities[_.upperFirst(relationship.otherEntityName)],
    to: configuration.jdl.entities[entityName],
    type: RelationshipTypes.ONE_TO_MANY,
    injectedFieldInFrom: relationshipConfiguration.injectedFieldInTo,
    injectedFieldInTo: getInjectedFieldInFrom(relationship),
    isInjectedFieldInFromRequired: relationshipConfiguration.isInjectedFieldInToRequired,
    isInjectedFieldInToRequired: relationship.relationshipValidateRules,
    commentInFrom: relationshipConfiguration.commentInTo,
    commentInTo: relationship.javadoc
  });
}

function getInjectedFieldInFrom(relationship) {
  return relationship.relationshipName
    + (relationship.otherEntityField && relationship.otherEntityField !== 'id'
      ? `(${relationship.otherEntityField})`
      : '');
}

function addEntityOptionsToJDL(entity, entityName) {
  if (entity.fluentMethods === false) {
    addUnaryOptionToJDL(UnaryOptions.NO_FLUENT_METHOD, entityName);
  }
  [BinaryOptions.DTO, BinaryOptions.PAGINATION, BinaryOptions.SERVICE, BinaryOptions.SEARCH_ENGINE].forEach((option) => {
    if (entity[option] && entity[option] !== 'no') {
      addBinaryOptionToJDL(option, entity[option], entityName);
    }
  });
  // angularSuffix in BinaryOptions, angularJSSuffix in Json
  if (entity.angularJSSuffix) {
    addBinaryOptionToJDL(BinaryOptions.ANGULAR_SUFFIX, entity.angularJSSuffix, entityName);
  }
  // microservice in BinaryOptions, microserviceName in Json
  if (entity.microserviceName !== undefined) {
    addBinaryOptionToJDL(BinaryOptions.MICROSERVICE, entity.microserviceName, entityName);
  }
  if (entity.jpaMetamodelFiltering === true) {
    addUnaryOptionToJDL(UnaryOptions.FILTER, entityName);
  }
  if (entity.clientRootFolder) {
    addBinaryOptionToJDL(BinaryOptions.CLIENT_ROOT_FOLDER, entity.clientRootFolder, entityName);
  }
}

function addUnaryOptionToJDL(unaryOption, entityName) {
  configuration.jdl.addOption(
    new JDLUnaryOption(
      {
        name: unaryOption,
        entityNames: [entityName]
      }
    )
  );
}

function addBinaryOptionToJDL(binaryOption, value, entityName) {
  configuration.jdl.addOption(
    new JDLBinaryOption(
      {
        name: binaryOption,
        value,
        entityNames: [entityName]
      }
    )
  );
}

/**
 * Parses the jhipster configuration into JDL.
 * @param jhConfig the jhipster config ('generator-jhipster' in .yo-rc.json)
 * @param jdl {JDLObject} to which the parsed options are added. If undefined a new JDLObject is created.
 * @returns {JDLObject} the JDLObject
 */
function parseServerOptions(jhConfig, jdl) {
  if (!jdl) {
    jdl = new JDLObject();
  }
  [UnaryOptions.SKIP_CLIENT, UnaryOptions.SKIP_SERVER, UnaryOptions.SKIP_USER_MANAGEMENT].forEach((option) => {
    if (jhConfig[option] === true) {
      jdl.addOption(new JDLUnaryOption({
        name: option,
        value: jhConfig[option]
      }));
    }
  });
  return jdl;
}
